# Solution Challenge SSTIC 2024


## Introduction

Ce rapport présente un petit résumé de ma solution pour le challenge SSTIC 2024.

## Prologue
### Enoncé

![Enocé prologue](image.png)


### Objectif
Le challenge commence par le téléchargement d'une archive contenant un rapport de police ainsi qu'une archive de base de données de Tesla qui contient tout un jeu de données comprenant toutes les dernières positions de la Tesla, ses lieux visités, l'état de la batterie... 

Le rapport de police nous fait comprendre qu'il faut retrouver les 5 lieux les plus visités par le propriétaire de la Tesla suspecte.

### Résolution :
Après avoir chargé la base dans postGRESQL et avoir lancé quelques recherches "à la main", on finit par retrouver des "indices": Une borne de recharge dont le nom commence en http:, des numéros de rue, un numéro de port en ":8080"... Après une grande réflexion et de grands calculs savants, j'ai fini par retrouver l'adresse IP en question (pas du tout, j'ai bruteforce...) et le flag !

![Flag prologue](<Screenshot from 2024-05-21 10-20-19.png>)


### Note

    J'aurais pu utiliser le super logiciel teslamate pour afficher de beaux graphiques Grafana et sortir la solution beaucoup plus facilement...

### Outils utilisés :

- Python
- postGRE-SQL

## Chapitre 1 : Attaque par Oracle de Padding

### Enoncé
![Enoncé étape 1](image-1.png)

### Objectif :

Dans ce niveau en tant qu'attaquant on se positionne en **man-in-the-middle** entre un **license server** et différents clients. L'objectif est de réussir à **"extraire" la clef de licence d'un utilisateur** depuis le serveur.

### Résolution :
On peut donc voir passer différentes trames qui paraissent chiffrées. On se rend assez rapidement compte que le chiffrement en question est un **chiffrement par bloc** et que l'on peut commencer à modifier quelques octets dans les trames, et notamment ceux de l'IV qui se situent au début du message chiffré, afin de mieux comprendre le protocole de communication...

On note également que lorsque l'on modifie des octets dans la suite du message, il arrive que le serveur nous réponde que le padding du message n'est pas conforme à la norme **ISO 7816-4**. Ce qui nous laisse penser qu'une attaque par oracle de padding(POA) serait possible par la suite afin de pouvoir aussi bien décoder qu'encoder des messages à la place d'un client.
Grâce aux messages d'erreur du serveur, on peut comprendre que le message commence par:
- un **IV**,
- suivi d'un **ID de session**,
- d'un **numéro de commande**,
- d'une **taille**,
- du **message**

        message_struct = Struct(
            "status" / Byte,
            "session_id" / Bytes(8),
            "command" / Byte,
            "size" / Int16ul,
            "message" / Bytes(lambda ctx: ctx.size)
        )

Il faut noter que l'attaque par oracle de padding est possible sur les messages des clients mais aussi sur les messages des serveurs même si cela peut s'avérer fastidieux de décrypter un message serveur de cette façon.

Heureusement, on finit par se rendre compte qu'il est possible de récupérer le clair des JSON contenu dans certains messages afin de récupérer leur contenu en retour de message d'erreur en injectant un octet non-ASCII dans celui-ci (possible du fait du chiffrement par bloc).


En étudiant par la suite les différentes commandes disponibles grâce à la POA, on se rend compte qu'il est possible de prendre la place d'un utilisateur une fois que ce dernier est connecté et d'envoyer une requête de licence au serveur afin que ce dernier nous affiche la clef de licence.


Attention à prendre la place d'un utilisateur enregistré, tous ne l'étant pas...


Finalement, on réussit à extraire la clef de licence,à la rentrer sur la page web du chapitre 1, ce qui nous amène par la suite au chapitre 2 et au flag du chapitre 1 !

![Flag étape 1](image-4.png)


### Note
    Je ne m'étendrai pas sur les optimisations dans l'ordre des suppositions de caractères selon la fréquence d'occurence dans la langue anglaise et des longues minutes passées à déchiffrer des messages serveur...

### Outils utilisés :
- Python

## Chapitre 2 : Exploitation

### Enoncé
![Enoncé étape 2](image-3.png)

### Objectif

Ce niveau consiste en un jeu sur lequel on peut se connecter grâce à la licence précédemment obtenue. Le but est de réussir à obtenir un **reverse shell** sur la machine d'un joueur qui est connecté sur le jeu mais qui semble AFK.
Pour cela on a en notre possession le script Python qui fait office de "serveur" et le client du jeu qui est exécuté pour lancer le jeu.

On comprend donc bien que l'on va devoir trouver une ou plusieurs vulnérabilités sur la machine de l'attaquant afin de pouvoir les exploiter.

### Vulnérabilités

Il n'y a pas énormément de rétro-ingénierie à effectuer, les concepteurs ont été gentils et nous ont donné un binaire avec tous les symboles. On repère assez vite une vulnérabilité de type **format string** qui nous permet de leak 2 pointeurs : Le premier dans la **libc**, le second dans la **stack**. Plutôt pratique ! La fonction utilisée pour la format string étant **snprintf_chk**, elle ne permet pas d'utiliser les chaines de format comme **%n** qui nous auraient donné une écriture... Il faut donc trouver une autre vuln.

C'est finalement en jouant que j'ai pu la trouver, en effet lorsqu'un joueur possède un bouclier et se fait pousser en dehors de la map, il perd son bouclier mais le pointeur vers ce dernier n'est pas remis à 0 et on se retrouve avec une valeur qui ressemble à un pointeur à la place de notre barre de vie.

Ce pointeur est en réalité un pointeur vers le maillon suivant de la free list dans lequel se retrouve le chunk venant d'être libéré et qui correspondait au bouclier.

Il est d'ailleurs possible de modifier cette valeur à distance en enlevant des points de vie au joueur qui vient de perdre son bouclier...

### Exploitation

L'exploitation se réalise en plusieurs étapes...

La première étape consiste à récupérer les différets leak (libc et stack).

Ensuite, il faut trig la vuln, faire passer le chunk des fastbins au tcache afin d'éviter les différents checks effectués.

Enfin, on peut modifier l'adresse du prochain chunk alloué vers la stack sur une adresse de retour de fonction afin d'initier une ropchain et de lancer un reverse shell Python avec la fonction SYSTEM.

![Flag étape 2](image-5.png)


### Outils utilisés :
- Python
- GDB/gef
- serveo.net

## Chapitre 3 : Attaque d'un HMAC-SHA256 par injection de faute

![Enoncé étape 3](image-6.png)

### Objectif

Pour ce niveau, le but est d'attaquer un HSM qui est en charge de signer des messages avec l'algorithme HMAC-SHA256 et sur lequel une "backdoor" a été placée. On a en notre possession les modifications qui correspondent à la backdoor et on se rend compte que cette backdoor nous permet en fait de choisir sur quel tour d'un SHA-256 injecter une faute et à quel round de ce SHA. La faute est injectée sur un registre de travail "c". Après un peu de recherche sur internet on retrouve assez rapidement un papier qui correspond en tout point avec ce qu'il nous est possible de faire. 

https://www.researchgate.net/publication/307694142_Algebraic_Fault_Attack_on_the_SHA-256_Compression_Function

### Résolution
Pour la résolution de cette étape, j'ai donc appliqué à la lettre les instructions de ce papier et je ne m'étendrai, dans ce rapport, que sur les différences que j'ai pu y apporter.

Comme mentionné dans le papier j'ai utilisé le solveur STP en Python qui, pour le coup, m'a semblé beaucoup plus efficace que Z3 sur ce genre de problèmes. La seule différence avec le papier est que j'ai ajouté un tour de solveur dans chaque attaque afin de n'avoir à écrire que les équations de transition entre les états du SHA256.

Après avoir joué l'attaque on se retrouve avec la possibilité d'ajouter des blocs à un message existant et on est capable de calculer sa signature. Cela nous permet d'atteindre une vulnérabilité au niveau du parsing du message et donc de réussir à envoyer un message à notre cible en tant que quelqu'un d'autre.

Afin de résoudre le challenge j'ai finalement réutilisé le client Python qui nous était fourni, sur lequel j'ai ajouté un bouton cliquable afin de "casser" le HMAC et j'ai utilisé proxychains afin de projeter ce client sur la machine distante au travers d'un tunnel SSH. Après avoir envoyé un message à notre cible on obtient une nouvelle adresse HTTP et le flag !

![Flag étape 3](image-7.png)

### Outils utilisés :
- Python + STP
- https://sha256algorithm.com/
- proxychains, SSH


## Chapitre 4: Exploitation module Google Chrome

### Enoncé

![Enoncé étape 4](image-8.png)

### Objectif

Le but de ce niveau est de réussir à exploiter une vulnérabilité dans un module Chrome.

### Exploitation

La vulnérabilité se voit assez vite étant donné le peu de code à notre disposition. On retrouve immédiatement une lecture relative grâce à la variable **saved_value** qu'il nous est possible d'afficher.

Pour l'écriture relative, on sent qu'il pourrait y avoir une race condition à trigger quand on regarde la fonction **try_authenticate**:
![Méthode try_authenticate](image-9.png)

Mais je n'aime pas les race conditions alors j'ai un peu fouillé et j'ai fini par trouver un moyen d'écrire 1 octet avec 7 octets d'overwrite, mais seulement vers des adresses inférieures. 

Pour cette écriture on va utiliser la fonction **try_autheticate** qui sauve une valeur, la patch, puis la restaure... L'idée est de modifier l'offset auquel la fonction applique le patch avant la restauration afin que cette dernière se fasse au véritable endroit souhaité. L'écriture se fait en 2 appels à **try_authenticate**.

Le premier appel a pour but d'écrire 1 octet (celui que l'on veut réellement écrire) juste à côté de l'endroit où est stockée la variable d'offset. Ainsi, lorsque l'on va venir réécrire à ce même endroit dans un second temps, la **saved_value** contiendra notre octet.

- Première étape on écrit 1 octet "dans le vide" et 7 octets d'offset afin de ne pas venir annuler ce que l'on vient d'écrire.
- Deuxième étape on réécrit au même endroit, la saved_value contient l'octet précédemment écrit, et avec les 7 octets de poids faible de l'offset on vient écrire l'offset relatif de là où on veut réellement écrire notre octet.

On ne peut donc écrire que vers des adresses inférieures.

La suite de l'exploitation consiste à retrouver l'endroit où se trouve notre buffer afin de pouvoir transformer notre écriture relative en écriture arbitraire.

Puis on trouve l'endroit du code afin de lire la liste des pages en RWX qui contient une page qui a été créée pour une fonction WASM que l'on a déclarée précédemment.

Finalement on écrit notre shellcode dans la page RWX, on appelle notre fonction WASM et le tour est joué(Le navigateur est lancé avec le **--no-sandbox**, ce qui rend l'exploitation relativement triviale une fois les primitives R/W obtenues). 

On obtient notre reverse Shell, une fois de plus grâce à **serveo** et on va lire une nouvelle adresse http dans un des fichiers présents sur la machine distante.

![Flag étape 4](image-10.png)


### Outils utilisés :
- Python
- VirtualBox
- WinDbg
- Chromium
- serveo.net


## Chapitre 5: Exploitation driver Windows

### Enoncé

![Enoncé étape 5](image-11.png)

### Objectif

L'objectif de ce niveau est simple, il faut élever ses privilèges en exploitant le driver Windows présenté dans l'énoncé.

### Reverse et recherche de vulnérabilité

On commence par reverse le driver afin de comprendre son fonctionnement et plus particulièrement la partie en charge de la gestion des ioctl. Le driver expose plusieurs ioctl qui nous permettent de :

- Créer un port en récepteur ou émetteur,
- Fermer un port,
- Envoyer des données immédiatement ou en "delayed",
- Recevoir des données

Ce sont uniquement ces 4 fonctions qui vont nous intéresser pour la suite. On n'a pas vraiment besoin de s'intéresser à la partie **Ndis** en charge du protocole de communication.

La vulnérabilité se trouve dans la fonction qui permet de créer un port. En effet, la fonction en charge de créer un port commence par vérifier que les flags indiquant si le port est émetteur ou récepteur ne sont pas tous les deux à 1. Puis, la fonction fait tout un tas de traitement, et enfin stocke la valeur passée par le programme userland.

Il y a donc une vulnérabilité de type TOCTOU qui nous permet de créer un port à la fois émetteur et récepteur.

Toute la suite de l'exploitation va donc se baser sur la confusion de type sur une liste qui dans le cas d'un port émetteur contient les messages en attente d'envoi et dans le cas d'un port récepteur contient les messages en attente d'être reçus. Dans un cas, les éléments de cette liste contiennent un offset dans une sorte de buffers circulaire, dans l'autre cas ce sont directement les données qui sont stockées.

Grâce à cette confusion on arrive donc à se fabriquer une lecture et une écriture relatives.

Pour le reste de l'exploitation, je me suis basé sur le github : https://github.com/vp777/Windows-Non-Paged-Pool-Overflow-Exploitation qui explique plutôt bien comment se fabriquer une lecture/écriture arbitraire en utilisant des named pipe.

Finalement, il ne reste plus qu'à retrouver le token SYSTEM et l'écrire à la place du token de notre process. On peut donc lancer un shell SYSTEM et aller récupérer dans les fichiers administrateurs le flag de cette dernière étape !

### Outils utilisés :
- Python
- VirtualBox
- WinDbg
- x86_64-w64-mingw32-g++

## Validation du challenge

Comme tous les ans, après avoir validé la dernière étape il ne nous reste plus qu'à retrouver l'adresse mail des organisateurs. Après avoir esquivé les multiples trolls à base de lobster dogs on finit par trouver un fichier audio qui émet des "WAF WAF" ainsi qu'un son assez singulier pendant quelques secondes.

En affichant le spectrogramme grâce à Audacity (merci chatGPT) on retrouve l'image suivante :

![Validation du challenge](image-12.png)


### Outils utilisés :
- Audacity

# Conclusion

J'ai trouvé le challenge SSTIC de cette année assez difficile du fait qu'il abordait beaucoup de sujets nouveaux pour moi. J'ai par contre beaucoup aimé les différents niveaux chacun avec leur spécificité. J'ai particulièrement apprécié le niveau 3 sur l'attaque du HMAC. 

La validation du challenge au dernier jour ne m'a pas permis de rendre ma solution à temps, mais il serait dommage de ne pas clore tout ce temps passé par un petit compte rendu...

Pour finir, je souhaiterais remercier les organisateurs du challenge ainsi que les différentes personnes avec qui j'ai pu échanger pendant mes longues heures de galère.